%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   This function eveluates period of nuclear NF-kB oscillations for      %
%   evety single cell considered. Inputs include: T- discretized          %
%   simulation time, X - nuclear NF-kB over the considered time course,   %
%   dt -simulation step and simulation chcracteristic times (t0, te).     %
%   Function returns a vector of the oscyllation periods (Per).           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Per]=findex(T,X,dt,t0,te)

tmax=[];           %vector defined to store time points at which nuclear NF-kB has a maxima
tmin=[];           %vector defined to store time points at which nuclear NF-kB has a minima
x=length(X);
a=0;b=0;c=0;
Per=[];
for i=2:(x-1)      %comparison to determine maximas and minimas in nuclear NF-kB
    a=X(i-1);
    b=X(i);
    c=X(i+1);
    if (a<b)&(b>c) tmax=[tmax,i];end;    %vector of time point at which maxima occured
    if (a>b)&(b<c) tmin=[tmin,i];end;    %vector of time point at which minima occured
end;
tmax=tmax(find((tmax-t0/dt)>0));         %time prior to TNf simulation (t0) is disregarded
tmin=tmin(find((tmin-t0/dt-dt)>0));      %time prior to TNf simulation (t0) is disregarded
tmin=[dt,tmin];                          %to have all maximas sourrounded by minima, 2 minimas are set artificially at the begining and end of simulation
if (tmax(length(tmax))>tmin(length(tmin))) tmin=[tmin,(te+t0)/dt];end;

% Based on all maximas and minimas, the proper maximas are determined
% 1st condition -- all proper maxima must be sourounded by a low minimas
D=0.75;            %minimas around proper maxima must be smaller than D fraction of a given maxima                                        
tp=[];             %stores proper maximas
marker=[];         %stores index of maximas which do not satisfy the 1st condition
for i=1:length(tmax)
    if (X(tmin(i))<D*X(tmax(i)))&(X(tmin(i+1))<D*X(tmax(i))) tp=[tp,tmax(i)];  %proper maximas
    else marker=[marker,i];end;                                                  
end;

% Maximas which do not satisfy  the 1st condition (indexed in marker) are %
% series of 2,3,4,... of consecutives maximas. Therefore as a proper      %
% maximas among them the largest among every series is taken              % 
TP=[];              %vector storing all proper maximas

% look for the possible series of consecutive four maximas in marker
if length(marker)>=4
   I=[];
   marker4=[];   % stores indexes of series of 4 consecutive maximas
   for i=1:length(marker)-3
      if (marker(i+1)==marker(i)+1)&(marker(i+2)==marker(i+1)+1)& (marker(i+3)==marker(i+2)+1) 
        marker4=[marker4,[marker(i),marker(i+1),marker(i+2),marker(i+3)]];
        I=[I,i];end;end;
   for j=1:length(I) marker(I(1,j)-(j-1)*4:I(1,j)+3-(j-1)*4)=[];end;  % found series of four are deleted from marker    
   for i=1:length(marker4)/4           % the largest of each four is a proper maxima
      t1=tmax(marker4(i+(i-1)*3));
      t2=tmax(marker4(i+1+(i-1)*3));
      t3=tmax(marker4(i+2+(i-1)*3));
      t4=tmax(marker4(i+3+(i-1)*3));
      if (X(t1)>X(t2))&(X(t1)>X(t3)&(X(t1)>X(t4))) TP=[TP,t1];end;
      if (X(t2)>X(t1))&(X(t2)>X(t3)&(X(t2)>X(t4))) TP=[TP,t2];end;
      if (X(t3)>X(t1))&(X(t3)>X(t2)&(X(t3)>X(t4))) TP=[TP,t3];end;
      if (X(t4)>X(t1))&(X(t4)>X(t2)&(X(t4)>X(t3))) TP=[TP,t4];end;    
    end;
end;

% look for the possible series of consecutive 3 maximas
if length(marker)>=3
I=[];
marker3=[];       % stores indexes of series of 3 consecutive maximas
for i=1:length(marker)-2
    if (marker(i+1)==marker(i)+1)&(marker(i+2)==marker(i+1)+1) 
        marker3=[marker3,[marker(i),marker(i+1),marker(i+2)]];
        I=[I,i];end;
end;
for j=1:length(I) marker(I(1,j)-(j-1)*3:I(1,j)+2-(j-1)*3)=[];end; % if found the series is deleted from marker
for i=1:length(marker3)/3                  % the largest of each three is a proper maxima
    t1=tmax(marker3(i+2*(i-1)));
    t2=tmax(marker3(i+1+2*(i-1)));
    t3=tmax(marker3(i+2+2*(i-1)));
    if (X(t1)>X(t2))&(X(t1)>X(t3)) TP=[TP,t1];end;
    if (X(t2)>X(t1))&(X(t2)>X(t3)) TP=[TP,t2];end;
    if (X(t3)>X(t1))&(X(t3)>X(t2)) TP=[TP,t3];end;
end;end;

% look for the possible series of consecutive 2 maximas
if length(marker)>=2
 I=[];
 marker2=[];
 for i=1:length(marker)-1
     if (marker(i+1)==marker(i)+1) 
         marker2=[marker2,[marker(i),marker(i+1)]];
         I=[I,i];end;
 end;
 for j=1:length(I) marker(I(1,j)-(j-1)*2:I(1,j)+1-(j-1)*2)=[];end; 
 for i=1:length(marker2)/2             % the largest of each two is a proper maxima
    t1=tmax(marker2(i+(i-1)));
    t2=tmax(marker2(i+1+(i-1)));
    if (X(t1)>X(t2)) TP=[TP,t1];end;
    if (X(t2)>X(t1)) TP=[TP,t2];end;
 end;end;

TP=sort([TP,tp]);          %all proper maxima stored        
Per=[TP(1)-t0/dt,TP(2)-TP(1),TP(3)-TP(2),TP(4)-TP(3)]; %periods evaluated
Per=Per*dt;                                            %periods in seconds